<?php
class AbstractController
{
	protected $assets;
	private $layoutName;

	public function isAjax()
	{
		return isset($_SERVER['HTTP_X_AJAX']);
	}

	public function __construct()
	{
		$this->switchLayout('layout-normal');

		$this->assets = new Assets();
		$this->assets->setTitle(Core::getConfig()->appearance->title);
	}

	public function renderAjax()
	{
		$this->switchLayout('layout-json');
		$this->renderView(null);
	}

	public function renderFile(FileRendererOptions $options)
	{
		$fileRenderer = new FileRenderer();
		$fileRenderer->render($options);
	}

	public function renderView($viewName)
	{
		$context = Core::getContext();
		if ($viewName !== null)
			$context->viewName = $viewName;
		View::renderTopLevel($this->layoutName, $this->assets);
	}


	protected function interceptErrors($action)
	{
		try
		{
			$action();
			return true;
		}
		catch (SimpleException $e)
		{
			\Chibi\Util\Headers::setCode(400);
			Messenger::fail($e->getMessage());
			return false;
		}
	}

	protected function redirectToLastVisitedUrl($filter = null)
	{
		$targetUrl = SessionHelper::getLastVisitedUrl($filter);
		if (!$targetUrl)
			$targetUrl = Core::getRouter()->linkTo(['StaticPagesController', 'mainPageView']);
		$this->redirect($targetUrl);
	}

	protected function redirect($url)
	{
		if ($this->isAjax())
		{
			Core::getContext()->transport->redirectUrl = $url;
			$this->renderAjax();
		}
		else
			\Chibi\Util\Url::forward($url);
	}


	private function switchLayout($layoutName)
	{
		$this->layoutName = $layoutName;
	}
}
